module ContextFreeGrammar exposing (..)

-- Symbol is a union type: terminals and nonterminals
-- type variable nt for "nonterminal type"
-- type variable tt for "terminal type"


type Symbol tt nt
    = Nonterminal nt
    | Terminal tt



-- wordform is a list of symbols, nonterminal or terminal, arbitrarily mixed


type alias Wordform tt nt =
    List (Symbol tt nt)


type alias Production tt nt =
    { leftSide : nt
    , rightSide : Wordform tt nt
    }


type alias ContextFreeGrammar tt nt =
    { startSymbol : nt
    , productions : List (Production tt nt)
    }


popKelements : Int -> List a -> ( List a, List a )
popKelements k list =
    if k <= 0 then
        ( [], list )

    else
        case list of
            [] ->
                ( [], [] )

            x :: rest ->
                let
                    ( topKminusOneElements, remainingElements ) =
                        popKelements (k - 1) rest
                in
                ( x :: topKminusOneElements, remainingElements )


symbolToString : Symbol tt nt -> String
symbolToString symbol =
    case symbol of
        Nonterminal x ->
            Debug.toString x

        Terminal x ->
            Debug.toString x
