module LR0Parser exposing (..)

import ContextFreeGrammar exposing (..)
import DKAutomaton exposing (..)
import ParseTree exposing (..)



{--
lr0parserToTheEnd : DKAutomaton tt nt state -> List tt -> List ( Symbol tt nt, state ) -> Bool
lr0parserToTheEnd dkAutomaton inputWord stack =
    if parsingCompleted dkAutomaton inputWord stack then
        True

    else
        let
            ( restInputWord, newStack, errorMessage ) =
                lr0parserStep dkAutomaton inputWord stack

            asdjfklajslkfdjaklsdjf =
                Debug.log ""
                    ("("
                        ++ stackToString (List.reverse newStack)
                        ++ ", "
                        ++ Debug.toString restInputWord
                        ++ ", "
                        ++ Maybe.withDefault "success" errorMessage
                        ++ ")"
                    )
        in
        case errorMessage of
            Just message ->
                False

            Nothing ->
                lr0parserToTheEnd dkAutomaton restInputWord newStack

--}
-- parsingCompleted checks whether stack has size 2 and, if yes, whether top symbol on stack is start symbol


parsingCompleted : DKAutomaton tt nt state -> List tt -> List ( Symbol tt nt, state ) -> Bool
parsingCompleted dkAutomaton inputWord stack =
    case ( stack, inputWord ) of
        ( [ ( topSymbol, topState ) ], [] ) ->
            topSymbol == Nonterminal dkAutomaton.startSymbol

        _ ->
            False



-- gets the top state of the stack; if stack is empty then it's the start state of the automaton


getTopState : DKAutomaton tt nt state -> List ( ParseTree tt nt, state ) -> state
getTopState dkAutomaton stack =
    case stack of
        ( topParseTree, topState ) :: _ ->
            topState

        [] ->
            dkAutomaton.startState


lr0parserStep : DKAutomaton tt nt state -> List tt -> List ( ParseTree tt nt, state ) -> ( List tt, List ( ParseTree tt nt, state ), Maybe String )
lr0parserStep dkAutomaton inputWord stack =
    {--
    case stack of
        [] ->
            ( inputWord, stack, Just "Stack is empty, no parse step performed." )

        ( topSymbol, currentState ) :: restStack ->
    --}
    let
        currentState : state
        currentState =
            getTopState dkAutomaton stack
    in
    case dkAutomaton.whichReduction currentState of
        Nothing ->
            -- we read the next input symbol and put it on the stack
            -- at the same time, we perform one DK-step and put the new state
            -- on top of the stack, together with the symbol just read
            --
            -- In the literature this is known as a SHIFT step
            case inputWord of
                nextTerminalSymbol :: restInputWord ->
                    let
                        -- we create a new tree consisting of a single terminal symbol
                        nextParseTree : ParseTree tt nt
                        nextParseTree =
                            Leaf nextTerminalSymbol

                        newState : state
                        newState =
                            dkAutomaton.delta currentState (Terminal nextTerminalSymbol)

                        newStack : List ( ParseTree tt nt, state )
                        newStack =
                            ( nextParseTree, newState ) :: stack
                    in
                    ( restInputWord, newStack, Nothing )

                -- the Nothing means everything is fine
                [] ->
                    ( inputWord, stack, Just "wanted to read but reached input end" )

        Just rule ->
            -- the DK automaton answered "yes, please use rule to reduce"
            let
                -- we must apply a reduction rule of the form X -> beta
                sizeOfRightside : Int
                sizeOfRightside =
                    List.length rule.rightSide

                -- this is the length of beta
                -- we now count the fact that beta lies on top of the stack
                ( poppedElements, stackAfterPopOperation ) =
                    popKelements sizeOfRightside stack

                -- poppedElements : List (ParseTree tt nt , State)
                poppedTrees : List (ParseTree tt nt)
                poppedTrees =
                    case List.map Tuple.first poppedElements of
                        [] ->
                            [ EpsilonLeaf ]

                        nonEmptyList ->
                            nonEmptyList

                topStateAfterPop =
                    getTopState dkAutomaton stackAfterPopOperation

                newState : state
                newState =
                    dkAutomaton.delta topStateAfterPop (Nonterminal rule.leftSide)

                newTree : ParseTree tt nt
                newTree =
                    Branch rule.leftSide (List.reverse poppedTrees)

                newStack : List ( ParseTree tt nt, state )
                newStack =
                    ( newTree, newState ) :: stackAfterPopOperation
            in
            ( inputWord, newStack, Nothing )



-- Nothing means everything is fine


stackToString : List ( ParseTree tt nt, state ) -> String
stackToString stack =
    String.join ", " (List.map stackCellToString stack)


stackCellToString : ( ParseTree tt nt, state ) -> String
stackCellToString ( parseTree, s ) =
    "(" ++ parseTreeToString parseTree ++ ", " ++ Debug.toString s ++ ")"
