module Tgrammar exposing (..)

import ContextFreeGrammar exposing (..)
import DKAutomaton exposing (..)


exampleInputWord =
    "aabbc"


startState =
    Eps


type Nonterminal
    = S
    | T


tGrammar : ContextFreeGrammar Char Nonterminal
tGrammar =
    { startSymbol = S
    , productions = [ rule1, rule2, rule3 ]
    }


rule1 : Production Char Nonterminal
rule1 =
    { leftSide = S, rightSide = [ Nonterminal T, Terminal 'c' ] }


rule2 : Production Char Nonterminal
rule2 =
    { leftSide = T, rightSide = [] }


rule3 : Production Char Nonterminal
rule3 =
    { leftSide = T
    , rightSide =
        [ Nonterminal T
        , Terminal 'a'
        , Nonterminal T
        , Terminal 'b'
        ]
    }


type State
    = Eps
    | Thut
    | Tc
    | Ta
    | TaT
    | TaTb
    | ErrState


tGrammarDKautomaton : DKAutomaton Char Nonterminal State
tGrammarDKautomaton =
    { startState = Eps
    , startSymbol = S
    , delta = delta
    , whichReduction = whichReduction
    }


dkAutomaton =
    tGrammarDKautomaton


delta : State -> Symbol Char Nonterminal -> State
delta state symbol =
    case ( state, symbol ) of
        ( ErrState, _ ) ->
            ErrState

        ( Eps, Nonterminal T ) ->
            Thut

        ( Thut, Terminal 'c' ) ->
            Tc

        ( Thut, Terminal 'a' ) ->
            Ta

        ( Ta, Nonterminal T ) ->
            TaT

        ( TaT, Terminal 'a' ) ->
            Ta

        ( TaT, Terminal 'b' ) ->
            TaTb

        _ ->
            ErrState


whichReduction : State -> Maybe (Production Char Nonterminal)
whichReduction state =
    case state of
        Eps ->
            Just rule2

        Ta ->
            Just rule2

        Tc ->
            Just rule1

        TaTb ->
            Just rule3

        _ ->
            Nothing
